package com.devsoap.tincheck.countries;

import com.devsoap.tincheck.tin.TinCountry;
import jakarta.inject.Singleton;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.ToString;
import org.apache.commons.lang3.StringUtils;

import java.time.LocalDate;
import java.util.Random;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

@Getter
@Singleton
@RequiredArgsConstructor
@ToString(of = {"countryCode", "countryName"})
public class Country_AT implements TinCountry {

    private static final int[] WEIGHTS = new int[]{1,2,1,2,1,2,1,2};

    private final String countryCode = "AT";
    private final String countryName = "Austria";

    @Override
    public String cleanTin(String tin) {
        return TinCountry.super.cleanTin(tin)
            .replace("/", "")
            .replace("-","");
    }

    @Override
    public boolean formatCheck(String tin) {
        return StringUtils.isNumeric(tin) && tin.length() == 9;
    }

    @Override
    public boolean logicCheck(String tin) {
        return extractDigits(tin)[8] == calculateCheckDigit(tin);
    }

    @Override
    public String generate(LocalDate yearOfBirth, Gender gender, Random randomGenerator) {
        boolean useFullForm = randomGenerator.nextBoolean();
        String tin;
        if (useFullForm) {
            tin = new StringBuilder()
                    .append(randomGenerator.ints(2,1,10)
                            .mapToObj(i -> i + "")
                            .collect(Collectors.joining()))
                    .append("-")
                    .append(randomGenerator.ints(3,1,10)
                            .mapToObj(i -> i + "")
                            .collect(Collectors.joining()))
                    .append("/")
                    .append(randomGenerator.ints(3,1,10)
                            .mapToObj(i -> i + "")
                            .collect(Collectors.joining()))
                    .toString();
        } else {
            tin = randomGenerator.ints(8,1,10)
                    .mapToObj(i -> i + "")
                    .collect(Collectors.joining());
        }
        return tin + calculateCheckDigit(cleanTin(tin));
    }

    private int calculateCheckDigit(String tin) {
        var digits = extractDigits(tin);
        var sum = IntStream.range(0,8)
                .map(i -> digits[i] * WEIGHTS[i])
                .map(n -> n > 9 ? (1 + n % 10) : n)
                .sum();
        return (100 - sum) % 10;
    }
}